/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include "ftp_main.h"

int ftp_filter_init() {
	filter_register("asciiconversion", FILTER_TYPE_DOWNSTREAM,
			filter_convert_to_ascii, 0,
			"filter for converting binary data to ascii");
	filter_register("asciiconversion", FILTER_TYPE_UPSTREAM,
			filter_convert_to_ascii, 0,
			"filter for converting binary data to ascii");
	return 0;
}

int filter_convert_to_ascii(struct transfer_t* t,
			    struct proxyinfo* pi,
			    struct filter_t* __unused__) {
	/* here we do non-ascii -> ascii - conversion, that means
	 * \n -> \r\n. That means that we _expand_ the transfer-buffer
	 * but there is no problem with overlapping transfer_t structs.
	 *
	 * we would have this problem if we did for example \r\n -> \n, because
	 * one buffer could end with \r and the other one start with \n.
	 * However this case does not exist in jftpgw because it reads always
	 * binary data XXX is this still true? */

	size_t new_size;
	int strictasciiconversion = 1;
	static int strictasciiconversion_set;

	if (pi->ci->data_connection.asciiconversion.serverlisting == 1) {
		/* no conversion in case of a server listing */
		jlog(9, "Serverlisting, filter_convert_to_ascii does no conversion");
		return 0;
	}

	/* don't get this value for each buffer, but for each transfer */
	if (!strictasciiconversion_set) {
		strictasciiconversion
				= config_get_bool("strictasciiconversion", 1);
	}

	if (t->buffer) {
		char* new_buffer;
		new_size = t->bufsize;
		new_buffer = to_ascii(t->buffer, &new_size,
							strictasciiconversion);
		free(t->buffer);
		t->buffer = new_buffer;
		t->bufsize = new_size;
	}

	/* reset after transfer */
	if (t->status == TRANSFER_STATUS_ERROR
				|| t->status == TRANSFER_STATUS_FINISHED) {
		strictasciiconversion_set = 0;
	}
	return 0;
}

