/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#ifndef __JPROXYGW_FTP_MAIN_H__
#define __JPROXYGW_FTP_MAIN_H__

#include "../libjproxygw/jproxygw.h"
#include "ftp_cmds.h"
#include "ftp_transfermode.h"
#include "ftp_events.h"
#include "ftp_types.h"
#include "ftp_socketio.h"
#include "ftp_capa.h"
#include "ftp_modules.h"
#include "fw_auth_cmds.h"
#include "ftp_filter.h"
#include "ftp_std.h"
#include "ftp_util.h"

/* modules */
#include "modules/ftp_cmdcheck.h"
#include "modules/ftp_spp.h"
#include "modules/ftp_squidcache.h"
#include "modules/ftp_diskcache.h"
#include "modules/ftp_nodatarelay.h"

#define DEFAULTSERVERPORT	21

/* see filter.h */
#define TRANSFER_DIRECTION_UPSTREAM	FILTER_TYPE_UPSTREAM
#define TRANSFER_DIRECTION_DOWNSTREAM	FILTER_TYPE_DOWNSTREAM
#define TRANSFER_DIRECTION_UNSPEC	(-1)

#define UNSPEC			0
#define PASSIVE			1
#define ACTIVE			2
#define ASCLIENT		3

#define CMD_HANDLED		0
#define CMD_DONE		1
#define CMD_QUIT		2
#define CMD_PREEXEC_OK		3
#define CMD_POSTEXEC_OK		4
#define CMD_ABORT		(-1)
#define CMD_ERROR		(-2)
#define CMD_NOTFOUND		(-3)
#define CMD_PREEXEC_FAIL	(-4)
#define CMD_NOT_HANDLED		(-5)

#define MAX_LINE_SIZE		4096

#define TRNSMT_SUCCESS		0
#define TRNSMT_ERROR		1
#define TRNSMT_ABORTED		2
#define TRNSMT_NOERRORMSG	4

/* login stages must be sorted numerically */
#define LOGIN_ST_NOT_CONNECTED	0
#define LOGIN_ST_CONNECTED	1
#define LOGIN_ST_USER		2
#define LOGIN_ST_LOGGEDIN	3
#define LOGIN_ST_FULL		4

struct clientinfo {
	const char* service;
	char* user;
	char* user_login;  /* the verbatim user string used */
	char* pass;
	char* destination;
	unsigned int destinationport;
	char* anon_user_pass;
	struct {
		int dataserversock;
		int dataclientsock;
		unsigned long int data_addr_to_server;
		unsigned long int data_addr_to_client;
		int *waitforconnect;
		unsigned int dataport;
		char* portcmd;
		/* direction:
		 * either TRANSFER_DIRECTION_UPSTREAM
		 * or     TRANSFER_DIRECTION_DOWNSTREAM
		 * or     TRANSFER_DIRECTION_UNSPEC */
		int direction;
		int complete;
		unsigned int transfer_duration;
		unsigned long int transferred;
		struct {
			int serverlisting;
			/* either 'i'mage or 'a'scii */
			char type;
		} asciiconversion;
		struct {
			int servermode;
			int clientmode;
		} transfermode;
	} data_connection;
	struct {
		const char* cmd;
		char* method;
		int send_or_retr;
		const char* filename;
		int respcode;
	} lastcmd;
	struct {
		char* msg_welcome;
		char* msg_after_user;
		char* msg_after_pass;
		int stage;		/* Takes LOGIN_ST_* values */
		struct {
			char* user;
			char* pass;
		} fw_auth;
	} login;
	struct {
		int active;
		char* accept_pw;
		char* send_pw;
		char* login; /* the login option used and activated for dest */
		int passauth;
		/* these are the values that get checked if we are in a 
		 * <forwarded> ... </forwarded> tag of the configuration file.
		 *
		 * original.user is the basic value */
		struct {
			char* user;
			unsigned long int dest_ip;
			char* destination;
			unsigned int destinationport;
		} original;
	} forward;
};

struct ftp_login_t {
	char* user;
	char* destination;
	int destinationport;
	int mode; /* 'a' or 'p' */
};

/* ftp_passive.c */
int pasvclient(struct proxyinfo*, struct messagebuffer*);
int pasvserver(struct proxyinfo*, struct messagebuffer*, struct ftp_message*);
void destroy_passive_portrange();

/* ftp_active.c */
int portcommandcheck(const char*, struct sockaddr_in*,
				struct proxyinfo*, struct messagebuffer*);
int activeclient(char*, struct proxyinfo*, struct messagebuffer*);
int activeserver(struct proxyinfo*, struct messagebuffer*, struct ftp_message*);
void destroy_active_portrange();

/* ftp_login.c */
int login(struct proxyinfo*, struct messagebuffer*, int);

/* ftp_cmds.c */
int handle_cmds(struct proxyinfo*, struct messagebuffer*);



#endif
