/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include <signal.h>
#include <ctype.h>
#include "ftp_main.h"

extern int timeout;

char* ftp_readline(int fd) {
	/* check for valid FTP responses */
	return readline_check(fd, 1);
}

/* ftp_readall() reads from an fd and returnes the whole data in a structure
 * message.
 */
struct ftp_message ftp_readall(int sourcefd) {
	char* line =0, *buf =0, *tmp =0, *last =0, *linestart =0;
	struct ftp_message ret = { (char*) 0,      /* msg */
				   0,              /* code */
				   0,              /* error */
				   ERROR_NONE,     /* error_type */
				   0,              /* errno_copy */
				   sourcefd        /* from_fd */
	};
	size_t tmpsize;

	while ((line = readline(sourcefd))) {
		if (buf) {
			tmpsize = strlen(buf) + strlen(line) + 4;
			tmp = (char*) malloc(tmpsize);
			enough_mem(tmp);
			strncpy(tmp, buf, tmpsize);
			last = tmp + strlen(buf);
			free(buf);
			buf = (char*) 0;
		} else {
			tmpsize = strlen(line) + 4;
			tmp = (char*) malloc(tmpsize);
			enough_mem(tmp);
			tmp[0] = '\0';
			last = tmp;
		}
		linestart = tmp + strlen(tmp);
		scnprintf(tmp, tmpsize, "%s\r\n", line);
		buf = tmp;
		tmp = (char*) 0;
		if (line[0] < '0' || line[0] > '9') {
			free(line);
			continue;
		}
		if (line[3] == ' ') {
			break;
		}
		free(line);
		line = (char*) 0;
	}
	if (!line) {
		free(buf);
		ret.error = 1;
		ret.errno_copy = errno;
		if (timeout) {
			ret.error_type = ERROR_TIMEOUT;
			/* XXX - better handling of this */
			timeout = 0;
		} else {
			/* XXX */
			ret.error_type = ERROR_CONNECTIONCLOSED;
		}
		return ret;
	} else {
		free(line);
	}
	jlog(9, "Readall (%d): %s", sourcefd, buf);
	ret.msg = buf;
	ret.code = respcode(last);
	ret.error = 0;
	ret.errno_copy = 0;
	return ret;
}


void ftp_message_say_error(struct ftp_message msg,
				struct messagebuffer* msgbuf, int to_fd,
				const char* filename, int lineno) {
	if (!msg.error) {
		return;
	}
	if (msg.error_type == ERROR_TIMEOUT) {
		/* Timeout */
		/* XXX ftp_socketio.c in debug string */
		message_buffer_addf(msgbuf, to_fd,
			500, "Connection timed out (ftp_socketio.c)\r\n");
		jlog(6, "Connection timed out contacting fd %d in "
				"file %s:%d", msg.from_fd, filename, lineno);
	} else if (msg.error_type == ERROR_CONNECTIONCLOSED) {
		/* Connection got closed */
		message_buffer_addf(msgbuf, to_fd,
			500, "Connection got closed\r\n");
		jlog(6, "Connection got closed on fd %d in "
				"file %s:%d", msg.from_fd, filename, lineno);
	}
}

