/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include "../ftp_main.h"
#include <ctype.h>

struct config_directives_list cmdcheck_configuration_directives_array[] = {
	{"enabled",		TAG_STARTUP, "no", EM, WSP },
	{"forbidden",		TAG_ALL, (char*) 0, EM, FL },
	{"allowed",		TAG_ALL, (char*) 0, EM, FL },
	{"maxcmdlength",	TAG_ALL, CHECK_CMD_MAXLEN, EM, WSP },
	{"mincmdlength",	TAG_ALL, CHECK_CMD_MINLEN, EM, WSP },
	{"argmaxlength",	TAG_ALL, CHECK_ARG_MAXLEN, EM, WSP },
	{"filemaxlength",	TAG_ALL, CHECK_ARG_FILE_MAXLEN, EM, WSP },
	{"usermaxlength",	TAG_ALL, CHECK_ARG_USER_MAXLEN, EM, WSP },
	{"passmaxlength",	TAG_ALL, CHECK_ARG_PASS_MAXLEN, EM, WSP },
	{"allowrelativepathnames",	TAG_ALL, "yes", EM, WSP },
	{ (char*) 0, 0, (char*) 0, 0, 0 }
};

struct config_namespace_directives_list cmdcheck_configuration_directives = {
	"cmdcheck",		/* namespace name */
	cmdcheck_configuration_directives_array
};

struct config_option_values_list cmdcheck_configuration_options_array[] = {
	{"enabled",			{ TRUEFALSE, TERM } },
	{"allowrelativepathnames",	{ TRUEFALSE, TERM } },
	{ TERM,         		{ TERM } }
};

struct config_namespace_options_list cmdcheck_configuration_options = {
	"cmdcheck",
	cmdcheck_configuration_options_array
};

int cmdcheck_add_configuration() {
	config_configuration_list_add(&cmdcheck_configuration_directives);
	config_options_list_add(&cmdcheck_configuration_options);
	return 0;
}

int cmdcheck_config_init() {
	config_register_hook(0, cmdcheck_add_configuration);
	return 0;
}

int cmdcheck_commandhandler_init() {
	/* register the ftp_check preexec */
	cmd_register(CMD_HANDLERTYPE_PREEXEC,
			CMD_SERIALIZE_REMOVE_NONE,
			"*", "FTP command check module", -1, -1,
			-1,
			ftp_cmdcheck,
			(char*) 0, (cmd_hookfunc_t) 0);
	return 0;
}

int cmdcheck_init() {
	module_hook_register(MODULE_HOOK_PROXY_INIT_CONFIG,
			     cmdcheck_config_init,
			     "cmdcheck config init");

	module_hook_register(MODULE_HOOK_FTP_LOGINTYPE_INIT,
			     cmdcheck_commandhandler_init,
			     "cmdcheck commandhandler init (for login)");

	module_hook_register(MODULE_HOOK_FTP_CMDHANDLER_INIT,
			     cmdcheck_commandhandler_init,
			     "cmdcheck commandhandler init");
	return 0;
}

static
struct ftpcheck_list_t ftpcheck_list[] = {
	{ "RETR", CHECK_FILENAME, 1, 1},
	{ "STOR", CHECK_FILENAME, 1, 1},
	{ "APPE", CHECK_FILENAME, 1, 1},
	{ "APPE", CHECK_FILENAME, 1, 1},
	{ "RNFR", CHECK_FILENAME, 1, 1},
	{ "RNTO", CHECK_FILENAME, 1, 1},
	{ "SIZE", CHECK_FILENAME, 1, 1},
	{ "STAT", CHECK_FILENAME, 1, 1},
	{ "DELE", CHECK_FILENAME, 1, 1},
	{ "MDTM", CHECK_FILENAME, 1, 1},

	{ "USER", CHECK_USERNAME, 1, 1},
	{ "ACCT", CHECK_USERNAME, 1, 1},

	/* allow empty passwords */
	{ "PASS", CHECK_PASS, 0, 1},

	{ "CWD",  CHECK_DIRECTORY, 1, 1},
	{ "XCWD", CHECK_DIRECTORY, 1, 1},
	{ "RMD",  CHECK_DIRECTORY, 1, 1},
	{ "XRMD", CHECK_DIRECTORY, 1, 1},
	{ "MKD",  CHECK_DIRECTORY, 1, 1},
	{ "XMKD", CHECK_DIRECTORY, 1, 1},

	{ "PORT", CHECK_ACTIVEPORT, 1, 1},

	{ "TYPE", CHECK_TYPE, 1, 1},

	{ "REST", CHECK_BYTEPOSITION, 1, 1},

	{ "CDUP", CHECK_NOTHING, 0, 0},
	{ "XCUP", CHECK_NOTHING, 0, 0},
	{ "QUIT", CHECK_NOTHING, 0, 0},
	{ "PASV", CHECK_NOTHING, 0, 0},
	{ "ABOR", CHECK_NOTHING, 0, 0},
	{ "PWD",  CHECK_NOTHING, 0, 0},
	{ "XPWD", CHECK_NOTHING, 0, 0},
	{ "SYST", CHECK_NOTHING, 0, 0},
	{ "NOOP", CHECK_NOTHING, 0, 0},

	{ "HELP", CHECK_HELP, 0, 1},

	/* the RFC does not specify that you can list multiple files like
	 *
	 * LIST file1 file2 file3 ...
	 *
	 * I did a test with proftpd and it seems to accept only one parameter.
	 * If you specify
	 *
	 * LIST file1 file2
	 *
	 * it tries to list "file1 file2"
	 */
	{ "LIST", CHECK_LISTARGS, 0, 1},
	{ "NLST", CHECK_NLSTARGS, 0, 1},

	/* we don't know about the maximum number of arguments. But let's limit
	 * it to 10 here. */
	{ "SITE", CHECK_SITECOMMAND, 0, 10},

	{ (char*) 0, 0 }
};


static
void check_generate_spacelist_option(const char* option_name,
					const char** value_p) {
	const char* val;

	ASSUME(value_p != NULL);
	val = *value_p;
	ASSUME(val != NULL);
	
	/* re-initialize the cmds_allowed-list */
	if (val[0] != ' ') {
		/* not yet initialized. The parser of the config file
		 * strips spaces */
		char* new, *formatted;

		new = strdup(val);
		enough_mem(new);

		replace_not_larger(new, "\t", " ");
		char_squeeze(new, ' ');
		toupstr(new);

		formatted = char_enclose(" ", new, " ");
		free(new);

		config_option_list_delete(option_name);
		config_option_list_add(option_name, formatted);
		free(formatted);
		*value_p = config_get_option(option_name);
	}
}

static
int check_help_ascii(const char* p, int start, int end, int should_log) {
	const char* orig = p;
	while (*p) {
		if (*p >= end || *p <= start) {
			if (should_log) {
				jlog(7, "ascii value mismatch in %s: %c (%d) (range %d - %d)",
					orig, *p, *p, start, end);
			}
			return CHECK_FAILED;
		}
		p++;
	}
	return CHECK_SUCCESS;
}

static
int check_help_number(const char* p) {
	const char* orig = p;
	while (*p) {
		if ( ! isdigit( (int) *p ) ) {
			jlog(7, "string that is supposed to be numeric contains non-digits: %s", orig);
			return CHECK_FAILED;
		}
		p++;
	}
	return CHECK_SUCCESS;
}

/* --------------------- general command checks ------------------- */

static
int check_forbidden(const char* cmd) {
	const char* cmds_forbidden = config_get_option("cmdcheck.forbidden");
	char* pattern;
	if (!cmds_forbidden) {
		return CHECK_SUCCESS;
	}
	check_generate_spacelist_option("cmdcheck.forbidden", &cmds_forbidden);

	ASSUME(cmd != NULL);
	ASSUME(cmds_forbidden != NULL);
	pattern = char_enclose(" ", cmd, " ");
	if (strstr(cmds_forbidden, pattern)) {
		/* found, but it is forbidden */
		free(pattern);
		return CHECK_FAILED;
	}
	free(pattern);
	return CHECK_SUCCESS;
}

/* the policy is:
 *
 *  - if cmdcheck.allowed is not defined, we continue checking
 *  - if cmdcheck.allowed is defined and the command in question is in there
 *    we accept
 *  - if it is not in there, we continue checking
 *
 * thus we can allow commands with this option. Any command not in there is
 * checked with its arguments according to the static table. We can forbid
 * commands regardless of whether they are in the static table or not with
 * the forbid option */

static
int check_allowed(const char* cmd) {
	const char* cmds_allowed = config_get_option("cmdcheck.allowed");
	char* pattern;
	if (!cmds_allowed) {
		return CHECK_DONTKNOW;
	}
	check_generate_spacelist_option("cmdcheck.allowed", &cmds_allowed);

	ASSUME(cmd != NULL);
	ASSUME(cmds_allowed != NULL);
	pattern = char_enclose(" ", cmd, " ");
	/* cmd is always in uppercase letters */
	if (strstr(cmds_allowed, pattern)) {
		/* found, it's allowed */
		free(pattern);
		return CHECK_SUCCESS;
	}
	free(pattern);
	return CHECK_DONTKNOW;
}

static
int check_maxcmdlength(const char* cmd) {
	int maxlength = config_get_ioption("cmdcheck.maxcmdlength", -1);

	ASSUME(maxlength > 0);
	if (strlen(cmd) > maxlength) {
		return CHECK_FAILED;
	}

	return CHECK_SUCCESS;
}

static
int check_mincmdlength(const char* cmd) {
	int minlength = config_get_ioption("cmdcheck.mincmdlength", -1);

	ASSUME(minlength > 0);
	if (strlen(cmd) < minlength) {
		return CHECK_FAILED;
	}
	return CHECK_SUCCESS;
}

static
int check_argmaxlength(const char* arg) {
	int maxlength = config_get_ioption("cmdcheck.argmaxlength", -1);

	if (!arg) {
		return CHECK_SUCCESS;
	}

	ASSUME(maxlength > 0);
	if (strlen(arg) > maxlength) {
		return CHECK_FAILED;
	}

	return CHECK_SUCCESS;
}

/* ------------------- checks of command classes ------------------- */

static
int check_filename(const char* arg) {
	int file_maxlen = config_get_ioption("cmdcheck.filemaxlength", -1);
	int allow_relative_pathnames 
		= config_get_bool("cmdcheck.allowrelativepathnames", 1);

	ASSUME(arg != NULL);
	ASSUME(file_maxlen > 0);
	if (strlen(arg) > file_maxlen) {
		jlog(7, "Filename too long: %s", arg);
		return CHECK_FAILED;
	}
	if (!allow_relative_pathnames) {
		if ((strstr(arg, "/../") || strncmp(arg, "../", 3) == 0)
			/* this would be an exception - it is a direct path,
			 * not a relative one. Remember: this option should NOT
			 * forbid directory traversal. */
			&& strcmp(arg, "../") != 0) {

			jlog(7, "relative pathnames forbidden: %s", arg);
			return CHECK_FAILED;
		}
	}
	return CHECK_SUCCESS;
}

static
int check_directory(const char* arg) {
	/* the same as the filename check right now */
	return check_filename(arg);
}

static
int check_username(const char* arg) {
	int user_maxlen = config_get_ioption("cmdcheck.usermaxlength", -1);

	ASSUME(arg != NULL);
	ASSUME(user_maxlen > 0);
	if (strlen(arg) > user_maxlen) {
		jlog(7, "Username too long: %s", arg);
		return CHECK_FAILED;
	}

	/* should only contain characters with ASCII code between 32 (space)
	 * and 126 (~) including @ (64) */
	if (check_help_ascii(arg, 32, 126, 1) != CHECK_SUCCESS) {
		return CHECK_FAILED;
	}
	return CHECK_SUCCESS;
}

static
int check_pass(const char* arg) {
	int pass_maxlen = config_get_ioption("cmdcheck.passmaxlength", -1);

	ASSUME(arg != NULL);
	ASSUME(pass_maxlen > 0);
	if (strlen(arg) > pass_maxlen) {
		jlog(7, "Password too long");
		return CHECK_FAILED;
	}
	/* should only contain characters with ASCII code between 32 (space)
	 * and 126 (~) including @ (64)
	 *
	 * the last parameter means: do not log the offending character and the
	 * string */
	if (check_help_ascii(arg, 32, 126, 0) != CHECK_SUCCESS) {
		return CHECK_FAILED;
	}
	return CHECK_SUCCESS;
}

static
int check_activeport(const char* arg) {
	struct sockaddr_in sin;
	int ret = parsesock(arg, &sin, ACTIVE);

	if (ret != 0) {
		return CHECK_FAILED;
	}
	return CHECK_SUCCESS;
}

static
int check_type(const char* arg) {
	ASSUME(arg != NULL);

	if (strlen(arg) != 1) {
		jlog(7, "type name has wrong size: %s", arg);
		return CHECK_FAILED;
	}

	if (arg[0] != 'I' && arg[0] != 'i'
	    && arg[0] != 'A' && arg[0] != 'a') {
		jlog(7, "type name is neither 'I' or 'A': %s", arg);
		return CHECK_FAILED;
	}

	return CHECK_SUCCESS;
}

static
int check_byteposition(const char* arg) {
	ASSUME(arg != NULL);

	/* all characters have to be numbers */
	if (check_help_number(arg) != CHECK_SUCCESS) {
		return CHECK_FAILED;
	}

	/* length:
	 5*1024*1024*1024*1024
	    TB   GB   MB   KB
	 5497558138880
	 1234567890123    5 TB has 13 digits. Want to transfer more every day?
	*/

	if (strlen(arg) > 13) {
		jlog(7, "byteposition too long: %s", arg);
		return CHECK_FAILED;
	}

	return CHECK_SUCCESS;
}

static
int check_help(const char* arg) {
	if (check_help_ascii(arg, 32, 126, 1) != CHECK_SUCCESS) {
		return CHECK_FAILED;
	}
	/* the argument to help should usually be a command, so that one should
	 * not be too long normally. However, I think CHECK_CMD_MAXLEN is too
	 * short and I don't see a reason why I should restrict it further than
	 * the "normal" constraints for arguments (CHECK_ARG_MAXLEN) that is
	 * checked anyway */
	return CHECK_SUCCESS;
}

static
int check_site(const char** args) {
	int i;
	for (i = 0; args[i]; i++) {
		if (check_help_ascii(args[i], 32, 126, 1) != CHECK_SUCCESS) {
			return CHECK_FAILED;
		}
	}
	/* The note above for check_help applies here in pretty much the same
	 * way... - but we can have multiple arguments here */
	return CHECK_SUCCESS;
}

static
int check_listargs(const char* arg) {
	/* normally this is -l or a filename, if at all.
	 * Check for a valid filename. That will apply the check for relative
	 * pathnames as well if the user wishes to forbid it */
	if (arg) {
		if (check_filename(arg) != CHECK_SUCCESS) {
			return CHECK_FAILED;
		}
	}
	return CHECK_SUCCESS;
}

static
int check_nlstargs(const char* arg) {
	/* the same as for listarg at the moment. Why should it be different?
	 * */
	return check_listargs(arg);
}

static
int check_cmd(const char* cmd,
		const char** args,
		int check_class,
		int nargs_min,
		int nargs_max) {

	int i;
	int ret;

	/* check the number of arguments */
	for (i = 0; i < nargs_min; i++) {
		if (!args[i]) {
			return CHECK_FAILED;
		}
	}
	for (i = 0; args[i]; i++) {
		if (i > nargs_max) {
			return CHECK_FAILED;
		}
	}

	ret = CHECK_FAILED;
	switch(check_class) {
		case CHECK_NOTHING:
			/* already checked - if there is an argument, but none
			 * is allowed */
			ret = CHECK_SUCCESS;
			break;
		case CHECK_FILENAME:
			ret = check_filename(args[0]);
			break;
		case CHECK_DIRECTORY:
			ret = check_directory(args[0]);
			break;
		case CHECK_USERNAME:
			ret = check_username(args[0]);
			break;
		case CHECK_PASS:
			ret = check_pass(args[0]);
			break;
		case CHECK_ACTIVEPORT:
			ret = check_activeport(args[0]);
			break;
		case CHECK_TYPE:
			ret = check_type(args[0]);
			break;
		case CHECK_BYTEPOSITION:
			ret = check_byteposition(args[0]);
			break;
		case CHECK_LISTARGS:
			ret = check_listargs(args[0]);
			break;
		case CHECK_NLSTARGS:
			ret = check_nlstargs(args[0]);
			break;
		case CHECK_SITECOMMAND:
			/* this command allows more than one parameter */
			ret = check_site(args);
			break;
		case CHECK_HELP:
			ret = check_help(args[0]);
			break;
	}
	return ret;
}

static
void cmdcheck_add_failmsg(struct proxyinfo* pi, struct messagebuffer* msgbuf) {

        int cs = pi->si->connection.clientsocket;

	message_buffer_add(msgbuf, cs,
			500, "Invalid command (cmdcheck)\r\n");
}

int ftp_cmdcheck(const char* cmd, const char** args,
		struct proxyinfo* pi, struct messagebuffer* msgbuf) {

	int ret;
	int i;
	int check_class, args_min, args_max;

	ASSUME(cmd != NULL);
	ASSUME(strchr(cmd, ' ') == (char*) 0);

	ret = check_allowed(cmd);
	if (ret == CHECK_FAILED) {
		cmdcheck_add_failmsg(pi, msgbuf);
		return CMD_PREEXEC_FAIL;
	}
	if (ret == CHECK_SUCCESS) {
		return CMD_PREEXEC_OK;
	}

	if (check_forbidden(cmd) != CHECK_SUCCESS) {
		cmdcheck_add_failmsg(pi, msgbuf);
		return CMD_PREEXEC_FAIL;
	}

	if (check_maxcmdlength(cmd) != CHECK_SUCCESS) {
		cmdcheck_add_failmsg(pi, msgbuf);
		return CMD_PREEXEC_FAIL;
	}

	if (check_mincmdlength(cmd) != CHECK_SUCCESS) {
		cmdcheck_add_failmsg(pi, msgbuf);
		return CMD_PREEXEC_FAIL;
	}

	if (check_argmaxlength(args[0]) != CHECK_SUCCESS) {
		cmdcheck_add_failmsg(pi, msgbuf);
		return CMD_PREEXEC_FAIL;
	}

	for(i = 0; ftpcheck_list[i].command != (char*) 0; i++) {
		if (strcmp(ftpcheck_list[i].command, cmd) == 0) {
			check_class = ftpcheck_list[i].handler;
			args_min = ftpcheck_list[i].nargs_min;
			args_max = ftpcheck_list[i].nargs_max;
			if ((ret = check_cmd(cmd, args, check_class,
					args_min, args_max)) != CHECK_SUCCESS) {

				cmdcheck_add_failmsg(pi, msgbuf);
				return CMD_PREEXEC_FAIL;
			}
			break;
		}
	}

	return CMD_PREEXEC_OK;
}

