/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#ifndef __FILTER_H__
#define __FILTER_H__

#include "jproxygw.h"

#define FILTER_TYPE_UPSTREAM		1
#define FILTER_TYPE_DOWNSTREAM		2

struct filter_t;

typedef int(*filter_func_t)(struct transfer_t*,
			    struct proxyinfo*,
			    struct filter_t*);

struct filter_t {
	char* name;
	filter_func_t filter_func;
	int type;
	int active;
	int rank;
	char* description;
};


/* ---- filter management functions ---- */

void filter_init(void);
void filter_deinit(void);
int filter_register(const char* name, int type,
			filter_func_t, int active, const char* description);

int filter_activate(const char* name, int type);
int filter_deactivate(const char* name, int type);

int filters_apply(int type, struct transfer_t* t, struct proxyinfo* pi);

void filters_sort(void);

/* ---- actual filters here ---- */

int filter_limit_bandwidth(struct transfer_t* t,
			   struct proxyinfo* __unused__,
			   struct filter_t* myself);

#endif /* __FILTER_H__ */
