/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#ifndef __JPROXYGW_TYPES_H__
#define __JPROXYGW_TYPES_H__

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


struct ilist_t {
	int value;
	struct ilist_t *next;
};

struct ullist_t {
	unsigned long int value;
	struct ullist_t *next;
};

struct slist_t {
	char* value;
	struct slist_t *next;
};

struct option_t {
	char* key;
	char* value;
	struct option_t* next;
};

struct option_prio_t {
	char* key;
	char* value;
	int priority;
	struct option_prio_t* next;
};

struct ip_t {
	unsigned long int ip;
	unsigned long int netmask;
	int error;
};

struct host_t {
	struct ip_t ip;
	char* name;
	int error;
};

struct address_t {
	struct host_t host;
	unsigned int port;
	int error;
};

struct hostent_list {
	char* name;
	unsigned long int ip;
	struct ullist_t* addr_list;
	struct slist_t* aliases_list;
	struct hostent_list* next;
	time_t lookup_time;
};

struct hostlist_t {
	struct host_t host;
	struct hostlist_t* next;
};

struct uidstruct {
	char* username;
	char* groupname;
	uid_t uid;
	gid_t gid;
};

struct connliststruct {
	pid_t pid;
	unsigned long int from_ip;
	unsigned long int proxy_ip;
	unsigned int      proxy_port;
	time_t            start_time;
	struct connliststruct* next;
};

struct limitstruct {
	struct ip_t ip;
	char* hostname;
	unsigned int connmax;
	unsigned int connected;
	struct connliststruct* connlist;
	struct limitstruct* next;
};

struct portrangestruct {
	unsigned int startport;
	unsigned int endport;
	struct portrangestruct* next;
};

struct messagebuffer {
	int fd;
	int code;
	char* msg;
	struct messagebuffer* next;
};


struct ilist_t* ilist_init(int);
void ilist_destroy(struct ilist_t*);
struct ilist_t* ilist_push(struct ilist_t*, int);
int ilist_pop(struct ilist_t*);
int ilist_empty(struct ilist_t* il);
struct ilist_t* ilist_clone(const struct ilist_t*);
/* --------end struct ilist-------------- */
/* --------begin struct ullist-------------- */
struct ullist_t* ullist_init(unsigned long int);
void ullist_destroy(struct ullist_t*);
struct ullist_t* ullist_push(struct ullist_t*, unsigned long int);
int ullist_pop(struct ullist_t*);
int ullist_empty(struct ullist_t*);
struct ullist_t* ullist_conv_ips(char**);
/* --------end struct ullist-------------- */
/* --------begin struct slist-------------- */
struct slist_t* slist_init(char*);
struct slist_t* slist_cinit(const char*);
void slist_destroy_free(struct slist_t*);
struct slist_t* slist_push(struct slist_t*, char*);
struct slist_t* slist_cpush(struct slist_t*, const char*);
struct slist_t* slist_append(struct slist_t*, struct slist_t*);
char* slist_get(struct slist_t*);
char* slist_pop(struct slist_t*);
int slist_empty(struct slist_t*);
struct slist_t* slist_conv_dpointer(char**);
struct slist_t* slist_clone(const struct slist_t*);
struct slist_t* slist_reverse(struct slist_t*);
char* slist_concat(struct slist_t*);
int slist_case_contains(const struct slist_t* haystack, const char* needle);
int slist_count(const struct slist_t*);
/* --------end struct slist-------------- */
/* --------begin struct optionlist-------------- */
struct option_t* optionlist_init(const char* key, const char* value);
struct option_t* optionlist_push(struct option_t*,
					const char* key,
					const char* value);
struct option_t* optionlist_clone(struct option_t*);
struct option_t* optionlist_append(struct option_t*, struct option_t*);
struct option_t* option_prio_list_sort(const struct option_prio_t* olist);
void optionlist_destroy(struct option_t*);
void optionlist_delete_key(struct option_t*, const char*);
const char* option_get_value(const struct option_t*, const char*);
/* --------end struct optionlist-------------- */
/* --------begin struct option_prio_list-------------- */
struct option_prio_t* option_prio_list_init(const char* key,
						const char* value,
						int line_number);
struct option_prio_t* option_prio_list_push(struct option_prio_t*,
					const char* key,
					const char* value,
					int line_number);
struct option_prio_t* option_prio_list_append(struct option_prio_t*,
						struct option_prio_t*);
struct option_prio_t* option_prio_list_clone(struct option_prio_t*);
void option_prio_list_destroy(struct option_prio_t*);
/* --------end struct option_prio_list-------------- */
/* --------begin struct hostent_list-------------- */
struct hostent_list* hostent_init(struct hostent*,
					unsigned long int ip,
					const char* name);
struct hostent_list* hostent_push(struct hostent_list*,
					struct hostent* e,
					unsigned long int ip,
					const char* name);
const struct hostent_list* hostent_get(struct hostent_list*,
				unsigned long int ip,
				const char* name);
const struct slist_t* hostent_get_aliases(struct hostent_list**,
					unsigned long int ip);
const struct ullist_t* hostent_get_addr(struct hostent_list**,
					const char* name);
const char* hostent_get_name(struct hostent_list**,
					unsigned long int ip);
unsigned long int hostent_get_ip(struct hostent_list**,
					const char* name);
void hostent_destroy(struct hostent_list*);
void hostent_delete(struct hostent_list* hl, const struct hostent_list* entry);
/* --------end struct hostent_list-------------- */
/* --------begin struct hostlist_t-------------- */
struct hostlist_t* hostlist_init();
struct hostlist_t* hostlist_push(struct hostlist_t**,
				 char* name,
				 unsigned long int ip,
				 unsigned long int netmask);
struct hostlist_t* hostlist_clone(struct hostlist_t*);
struct hostlist_t* hostlist_ip_push(struct hostlist_t**,
				    unsigned long int ip,
				    unsigned long int netmask);
struct hostlist_t* hostlist_name_push(struct hostlist_t**,
				      char* name);
/* --------end struct hostlist_t---------------- */
/* --------begin struct portrangestruct-------------- */
struct portrangestruct* portrangestruct_port2portrange(unsigned int port);
unsigned int portrangestruct_count(const struct portrangestruct*);
struct portrangestruct* portrangestruct_clone(struct portrangestruct*);
/* --------end struct portrangestruct---------------- */
/* --------start struct ip--------------------------- */
struct ip_t ip_parse_from_char(const char* str);
struct ip_t ip_from_s_addr(unsigned int s_addr);
char* ip_to_mchar(struct ip_t ip);
/* --------end struct ip----------------------------- */
/* --------start struct address---------------------- */
char* address_ip_mchar(struct address_t address);
unsigned int address_port(struct address_t address);
unsigned int address_port_hs(struct address_t address);
unsigned int address_port_ns(struct address_t address);
char* address_host_pref_name_mchar(struct address_t address);
char* address_ip_c_port_mchar(struct address_t address);
struct address_t address_parse_host_c_port(const char* str);
void address_free(struct address_t address);
/* ----------end struct address---------------------- */
/* --------start struct messagebuffer---------------- */
void message_buffer_initialize(struct messagebuffer* msgbuf);
void message_buffer_reset(struct messagebuffer* msgbuf);
void message_buffer_add_nomalloc(struct messagebuffer* msgbuf,
		       int fd, int code, char* msg);
void message_buffer_add(struct messagebuffer* msgbuf,
		       int fd, int code, const char* msg);
void message_buffer_addf(struct messagebuffer* msgbuf,
			 int fd, int code, const char* fmt, ...);
char* message_buffer_getmsg_m(const struct messagebuffer* msgbuf, int fd,
			int* respcode);
int message_buffer_say_reset(struct messagebuffer* msgbuf, int fd,
			int* respcode);
/* ----------end struct messagebuffer---------------- */

const char* gethostentip(const char*);


#endif
